@@ -86,7 +86,8 @@ data:
 import logging
 from dataclasses import dataclass, field
 from datetime import datetime as dt
-from typing import Any
+from pathlib import Path
+from typing import Any, Optional
 from warnings import warn
 
 import numpy as np
@@ -94,7 +95,7 @@ import pandas as pd  # type: ignore
 import xtgeo  # type: ignore
 
 from ._definitions import _ValidFormats
-from ._utils import generate_description
+from ._utils import generate_description, parse_timedata
 
 try:
     import pyarrow as pa  # type: ignore
@@ -118,27 +119,29 @@ class _ObjectDataProvider:
 
     * Investigating (parsing) the object (e.g. a XTGeo RegularSurface) itself
     * Combine the object info with user settings, globalconfig and class variables
+    * OR
+    * investigate current metadata if that is provided
     """
 
-    # input fields, cannot be defaulted
+    # input fields
     obj: Any
     dataio: Any
+    meta_existing: Optional[dict] = None
 
     # result properties; the most important is metadata which IS the 'data' part in
     # the resulting metadata. But other variables needed later are also given
     # as instance properties in addition (for simplicity in other classes/functions)
-    metadata: dict = field(default_factory=dict)
-
-    name: str = ""
-    classname: str = ""
-    efolder: str = ""
-    fmt: str = ""
-    extension: str = ""
-    layout: str = ""
-    bbox: dict = field(default_factory=dict)
-    specs: dict = field(default_factory=dict)
-    time0: str = ""
-    time1: str = ""
+    metadata: dict = field(default_factory=dict, init=False)
+    name: str = field(default="", init=False)
+    classname: str = field(default="", init=False)
+    efolder: str = field(default="", init=False)
+    fmt: str = field(default="", init=False)
+    extension: str = field(default="", init=False)
+    layout: str = field(default="", init=False)
+    bbox: dict = field(default_factory=dict, init=False)
+    specs: dict = field(default_factory=dict, init=False)
+    time0: str = field(default="", init=False)
+    time1: str = field(default="", init=False)
 
     def __post_init__(self):
 
@@ -481,11 +484,11 @@ class _ObjectDataProvider:
 
     def _derive_timedata_legacy(self):
         """Format input timedata to metadata. legacy version."""
-
         tdata = self.dataio.timedata
 
         tresult = dict()
-        if len(tdata) >= 1:
+        tresult["time"] = list()
+        if len(tdata) == 1:
             elem = tdata[0]
             tresult["time"] = list()
             xfield = {"value": dt.strptime(str(elem[0]), "%Y%m%d").isoformat()}
@@ -494,20 +497,39 @@ class _ObjectDataProvider:
                 xfield["label"] = elem[1]
             tresult["time"].append(xfield)
         if len(tdata) == 2:
-            elem = tdata[1]
-            xfield = {"value": dt.strptime(str(elem[0]), "%Y%m%d").isoformat()}
-            self.time1 = str(elem[0])
-            if len(elem) == 2:
-                xfield["label"] = elem[1]
-                self.time0 = str(elem[1])
-            tresult["time"].append(xfield)
+
+            elem1 = tdata[0]
+            xfield1 = {"value": dt.strptime(str(elem1[0]), "%Y%m%d").isoformat()}
+            if len(elem1) == 2:
+                xfield1["label"] = elem1[1]
+
+            elem2 = tdata[1]
+            xfield2 = {"value": dt.strptime(str(elem2[0]), "%Y%m%d").isoformat()}
+            if len(elem2) == 2:
+                xfield2["label"] = elem2[1]
+
+            if xfield1["value"] < xfield2["value"]:
+                tresult["time"].append(xfield1)
+                tresult["time"].append(xfield2)
+            else:
+                tresult["time"].append(xfield2)
+                tresult["time"].append(xfield1)
+
+            self.time0 = tresult["time"][0]["value"]
+            self.time1 = tresult["time"][1]["value"]
 
         logger.info("Timedata: time0 is %s while time1 is %s", self.time0, self.time1)
         return tresult
 
     def _derive_timedata_newformat(self):
-        """Format input timedata to metadata, new format."""
+        """Format input timedata to metadata, new format.
 
+        When using two dates, input convention is [[newestdate, "monitor"], [oldestdate,
+        "base"]] but it is possible to turn around. But in the metadata the output t0
+        shall always be older than t1 so need to check, and by general rule the file
+        will be some--time1_time0 where time1 is the newest (unless a class variable is
+        set for those who wants it turned around).
+        """
         tdata = self.dataio.timedata
         tresult = dict()
 
@@ -520,26 +542,54 @@ class _ObjectDataProvider:
                 xfield["label"] = elem[1]
             tresult["t0"] = xfield
         if len(tdata) == 2:
-            elem = tdata[1]
-            xfield = {"value": dt.strptime(str(elem[0]), "%Y%m%d").isoformat()}
-            self.time1 = str(elem[0])
-            if len(elem) == 2:
-                xfield["label"] = elem[1]
-            tresult["t0"] = xfield
+            elem1 = tdata[0]
+            xfield1 = {"value": dt.strptime(str(elem1[0]), "%Y%m%d").isoformat()}
+            if len(elem1) == 2:
+                xfield1["label"] = elem1[1]
 
-            elem = tdata[0]
-            xfield = {"value": dt.strptime(str(elem[0]), "%Y%m%d").isoformat()}
-            self.time0 = str(elem[0])
-            if len(elem) == 2:
-                xfield["label"] = elem[1]
-            tresult["t1"] = xfield
+            elem2 = tdata[1]
+            xfield2 = {"value": dt.strptime(str(elem2[0]), "%Y%m%d").isoformat()}
+            if len(elem2) == 2:
+                xfield2["label"] = elem2[1]
+
+            if xfield1["value"] < xfield2["value"]:
+                tresult["t0"] = xfield1
+                tresult["t1"] = xfield2
+            else:
+                tresult["t0"] = xfield2
+                tresult["t1"] = xfield1
+
+            self.time0 = tresult["t0"]["value"]
+            self.time1 = tresult["t1"]["value"]
 
         logger.info("Timedata: time0 is %s while time1 is %s", self.time0, self.time1)
         return tresult
 
+    def _derive_from_existing(self):
+        """Derive from existing metadata."""
+
+        # do not change any items in 'data' block, as it may ruin e.g. stratigrapical
+        # setting (i.e. changing data.name is not allowed)
+        self.metadata = self.meta_existing["data"]
+        self.name = self.meta_existing["data"]["name"]
+
+        # derive the additional attributes needed later e.g. in Filedata provider:
+        relpath = Path(self.meta_existing["file"]["relative_path"])
+        self.efolder = relpath.parent.name
+        self.classname = self.meta_existing["class"]
+        self.extension = relpath.suffix
+        self.fmt = self.meta_existing["data"]["format"]
+
+        self.time0, self.time1 = parse_timedata(self.meta_existing["data"])
+
     def derive_metadata(self):
         """Main function here, will populate the metadata block for 'data'."""
         logger.info("Derive all metadata for data object...")
+
+        if self.meta_existing:
+            self._derive_from_existing()
+            return
+
         nameres = self._derive_name_stratigraphy()
         objres = self._derive_objectdata()
 
@@ -552,6 +602,14 @@ class _ObjectDataProvider:
         meta["top"] = nameres.get("top", None)
         meta["base"] = nameres.get("base", None)
         meta["content"] = self.dataio._usecontent
+
+        # Outgoing content is always a string, but it can be given as a dict if content-
+        # specific information is to be included in the metadata.
+        # In that case, it shall be inserted in the data block as a key with name as the
+        # content, e.g. "seismic" or "field_outline"
+        if self.dataio._content_specific is not None:
+            meta[self.dataio._usecontent] = self.dataio._content_specific
+
         meta["tagname"] = self.dataio.tagname
         meta["format"] = objres["fmt"]
         meta["layout"] = objres["layout"]
@@ -560,6 +618,7 @@ class _ObjectDataProvider:
         meta["depth_reference"] = list(self.dataio.vertical_domain.values())[0]
         meta["spec"] = objres["spec"]
         meta["bbox"] = objres["bbox"]
+        meta["undef_is_zero"] = self.dataio.undef_is_zero
 
         # timedata:
         tresult = self._derive_timedata()
