@@ -13,11 +13,11 @@ from typing import Any, ClassVar, List, Optional, Tuple, Union
 from warnings import warn
 
 import pandas as pd  # type: ignore
-import yaml
 
 from . import _metadata
 from ._definitions import ALLOWED_CONTENTS, ALLOWED_FMU_CONTEXTS, CONTENTS_REQUIRED
 from ._utils import (
+    create_symlink,
     detect_inside_rms,
     drop_nones,
     export_file_compute_checksum_md5,
@@ -25,9 +25,9 @@ from ._utils import (
     filter_validate_metadata,
     generate_description,
     prettyprint_dict,
-    some_config_from_env,
-    uuid_from_string,
 )
+from ._utils import read_metadata as _utils_read_metadata
+from ._utils import some_config_from_env, uuid_from_string
 
 INSIDE_RMS = detect_inside_rms()
 
@@ -106,7 +106,6 @@ def _check_content(proposed: Union[str, dict]) -> Any:
     content = proposed
     logger.debug("content is %s of type %s", str(content), type(content))
     usecontent = "unset"
-    useextra = None
     if content is None:
         warn(
             "The <content> is not provided which defaults to 'depth'. "
@@ -116,14 +115,27 @@ def _check_content(proposed: Union[str, dict]) -> Any:
         usecontent = "depth"
 
     elif isinstance(content, str):
+        logger.debug("content is a string")
         if content in CONTENTS_REQUIRED:
             raise ValidationError(f"content {content} requires additional input")
         usecontent = content
+        content_specific = None  # not relevant when content is a string
+        logger.debug("usecontent is %s", usecontent)
 
     elif isinstance(content, dict):
+        logger.debug("content is a dictionary")
         usecontent = (list(content.keys()))[0]
-        useextra = content[usecontent]
-
+        logger.debug("usecontent is %s", usecontent)
+        content_specific = content[usecontent]
+        logger.debug("content_specific is %s", content_specific)
+        if not isinstance(content_specific, dict):
+            raise ValueError(
+                "Content is incorrectly formatted. When giving content as a dict, "
+                "it must be formatted as:"
+                "{'mycontent': {extra_key: extra_value} where mycontent is a string "
+                "and in the list of valid contents, and extra keys in associated "
+                " dictionary must be valid keys for this content."
+            )
     else:
         raise ValidationError("The 'content' must be string or dict")
 
@@ -134,12 +146,12 @@ def _check_content(proposed: Union[str, dict]) -> Any:
         )
 
     logger.debug("outgoing content is set to %s", usecontent)
-    if useextra:
-        _content_validate(usecontent, useextra)
-        return {usecontent: useextra}
+    if content_specific:
+        _content_validate(usecontent, content_specific)
     else:
         logger.debug("content has no extra information")
-        return usecontent
+
+    return usecontent, content_specific
 
 
 def _content_validate(name, fields):
@@ -194,15 +206,7 @@ def read_metadata(filename: Union[str, Path]) -> dict:
     Returns:
         A dictionary with metadata read from the assiated metadata file.
     """
-    fname = Path(filename)
-    metafile = str(fname.parent) + "/." + fname.stem + fname.suffix + ".yml"
-    metafilepath = Path(metafile)
-    if not metafilepath.exists():
-        raise IOError(f"Cannot find requested metafile: {metafile}")
-    with open(metafilepath, "r") as stream:
-        metacfg = yaml.safe_load(stream)
-
-    return metacfg
+    return _utils_read_metadata(filename)
 
 
 # ======================================================================================
@@ -277,11 +281,12 @@ class ExportData:
 
         casepath: To override the automatic and actual ``rootpath``. Absolute path to
             the case root. If not provided, the rootpath will be attempted parsed from
-            the file structure or by other means. Use with care!
+            the file structure or by other means. See also fmu_context, where "case"
+            may need an explicit casepath!
 
         config: Required, either as key (here) or through an environment variable.
-            A dictionary with static settings. In the standard case this is read
-            from FMU global variables (via fmuconfig). The dictionary must contain some
+            A dictionary with static settings. In the standard case this is read from
+            FMU global variables (via fmuconfig). The dictionary must contain some
             predefined main level keys to work with fmu-dataio. If the key is missing or
             key value is None, then it will look for the environment variable
             FMU_GLOBAL_CONFIG to detect the file. If no success in finding the file, a
@@ -294,9 +299,13 @@ class ExportData:
 
         fmu_context: In normal forward models, the fmu_context is ``realization`` which
             is default and will put data per realization. Other contexts may be ``case``
-            which will put data relative to the case root. If a non-FMU run is detected
-            (e.g. you run from project), fmu-dataio will detect that and set actual
-            context to None as fall-back.
+            which will put data relative to the case root (see also casepath). Another
+            important context is "preprocessed" which will output to a dedicated
+            "preprocessed" folder instead, and metadata will be partially re-used in
+            an ERT model run. If a non-FMU run is detected (e.g. you run from project),
+            fmu-dataio will detect that and set actual context to None as fall-back
+            (unless preprocessed is specified). If value is "preprosessed", see also
+            ``resuse_metadata`` key.
 
         description: A multiline description of the data either as a string or a list
             of strings.
@@ -305,12 +314,14 @@ class ExportData:
 
         forcefolder: This setting shall only be used as exception, and will make it
             possible to output to a non-standard folder. A ``/`` in front will indicate
-            an absolute path*; otherwise it will be relative to casepath or rootpath,
-            as dependent on the both fmu_context and the is_observations
-            boolean value. A typical use-case is forcefolder="seismic" which will
-            replace the "cubes" standard folder for Cube output with "seismics".
-            Use with care and avoid if possible! (*) For absolute paths, the class
-            variable allow_forcefolder_absolute must set to True.
+            an absolute path*; otherwise it will be relative to casepath or rootpath, as
+            dependent on the both fmu_context and the is_observations boolean value. A
+            typical use-case is forcefolder="seismic" which will replace the "cubes"
+            standard folder for Cube output with "seismics". Use with care and avoid if
+            possible! (*) For absolute paths, the class variable
+            allow_forcefolder_absolute must set to True.
+
+        grid_model: Currently allowed but planned for deprecation
 
         include_index: This applies to Pandas (table) data only, and if True then the
             index column will be exported. Deprecated, use class variable
@@ -319,7 +330,9 @@ class ExportData:
         is_prediction: True (default) if model prediction data
 
         is_observation: Default is False. If True, then disk storage will be on the
-            "share/observations" folder, otherwise on share/result
+            "share/observations" folder, otherwise on share/result. An exception arise
+            if fmu_context is "preprocessed", then the folder will be set to
+            "share/processed" irrespective the value of is_observation.
 
         name: Optional but recommended. The name of the object. If not set it is tried
             to be inferred from the xtgeo/pandas/... object. The name is then checked
@@ -335,6 +348,11 @@ class ExportData:
             detected automatically from the FMU run. Can be used to override in rare
             cases. If so, numbers must be >= 0
 
+        reuse_metadata_rule: This input is None or a string describing rule for reusing
+            metadata. Default is None, but if the input is a file string or object with
+            already valid metdata, then it is assumed to be "preprocessed", which
+            merges the metadata after predefined rules.
+
         runpath: TODO! Optional and deprecated. The relative location of the current run
             root. Optional and will in most cases be auto-detected, assuming that FMU
             folder conventions are followed. For an ERT run e.g.
@@ -360,6 +378,8 @@ class ExportData:
 
         workflow: Short tag desciption of workflow (as description)
 
+        undef_is_zero: Flags that nans should be considered as zero in aggregations
+
 
     .. note:: Comment on time formats
 
@@ -375,9 +395,11 @@ class ExportData:
 
             data:
               t0:
-                value: 2018010T00:00:00 description: base
+                value: 2018010T00:00:00
+                description: base
               t1:
-                value: 202020101T00:00:00 description: monitor
+                value: 202020101T00:00:00
+                description: monitor
 
         The output files will be on the form: somename--t1_t0.ext
 
@@ -415,6 +437,7 @@ class ExportData:
     case_folder: ClassVar[str] = "share/metadata"
     createfolder: ClassVar[bool] = True
     cube_fformat: ClassVar[str] = "segy"
+    filename_timedata_reverse: ClassVar[bool] = False  # reverse order output file name
     grid_fformat: ClassVar[str] = "roff"
     include_ert2jobs: ClassVar[bool] = False  # if True, include jobs.json from ERT2
     legacy_time_format: ClassVar[bool] = False
@@ -437,11 +460,14 @@ class ExportData:
     description: Union[str, list] = ""
     fmu_context: str = "realization"
     forcefolder: str = ""
+    grid_model: Optional[str] = None
     is_observation: bool = False
     is_prediction: bool = True
     name: str = ""
+    undef_is_zero: bool = False
     parent: str = ""
     realization: int = -999
+    reuse_metadata_rule: Optional[str] = None
     runpath: Union[str, Path, None] = None
     subfolder: str = ""
     tagname: str = ""
@@ -494,30 +520,37 @@ class ExportData:
             self.config = some_config_from_env(GLOBAL_ENVNAME)
 
         self._validate_content_key()
+        logger.info("Validate FMU context which is %s", self.fmu_context)
         self._validate_fmucontext_key()
         self._update_globalconfig_from_settings()
         _check_global_config(self.config, strict=True)
         self._establish_pwd_rootpath()
 
         self._show_deprecations_or_notimplemented()
+        logger.info("FMU context is %s", self.fmu_context)
         logger.info("Ran __post_init__")
 
     def _show_deprecations_or_notimplemented(self):
-        """Warn on deprecated keys og on stuff not implemented yet."""
+        """Warn on deprecated keys or on stuff not implemented yet."""
 
         if self.runpath:
             warn(
                 "The 'runpath' key has currently no function. It will be evaluated for "
-                "removal in fmu-dataio version 1. Use 'casepath' instead!",
+                "removal in fmu-dataio version 2. Use 'casepath' instead!",
+                PendingDeprecationWarning,
+            )
+
+        if self.grid_model:
+            warn(
+                "The 'grid_model' key has currently no function. It will be evaluated "
+                "for removal in fmu-dataio version 2.",
                 PendingDeprecationWarning,
             )
 
     def _validate_content_key(self):
         """Validate the given 'content' input."""
 
-        updated_content = _check_content(self.content)
-        # keep the updated content as extra setting
-        self._usecontent = updated_content
+        self._usecontent, self._content_specific = _check_content(self.content)
 
     def _validate_fmucontext_key(self):
         """Validate the given 'fmu_context' input."""
@@ -554,6 +587,7 @@ class ExportData:
         self._show_deprecations_or_notimplemented()
         self._validate_content_key()
         self._validate_fmucontext_key()
+        logger.info("Validate FMU context which is now %s", self.fmu_context)
 
     def _update_globalconfig_from_settings(self):
         """A few user settings may update/append the global config directly."""
@@ -611,6 +645,19 @@ class ExportData:
         logger.info("pwd:        %s", str(self._pwd))
         logger.info("rootpath:   %s", str(self._rootpath))
 
+    def _check_obj_if_file(self, obj: Any) -> Any:
+        """When obj is file-like, it must be checked + assume preprocessed."""
+
+        if isinstance(obj, (str, Path)):
+            if isinstance(obj, str):
+                obj = Path(obj)
+            if not obj.exists():
+                raise ValidationError(f"The file {obj} does not exist.")
+            if not self.reuse_metadata_rule:
+                self.reuse_metadata_rule = "preprocessed"
+
+        return obj
+
     # ==================================================================================
     # Public methods:
     # ==================================================================================
@@ -624,6 +671,9 @@ class ExportData:
         Examples of such known types are XTGeo objects (e.g. a RegularSurface),
         a Pandas Dataframe, a PyArrow table, etc.
 
+        If the key ``reuse_metadata_rule`` is applied with legal value, the object may
+        also be a reference to a file with existing metadata which then will be re-used.
+
         Args:
             obj: XTGeo instance, a Pandas Dataframe instance or other supported object.
             compute_md5: If True, compute a MD5 checksum for the exported file.
@@ -639,10 +689,12 @@ class ExportData:
             a temporary file, which may be time-consuming if the file is large.
         """
         logger.info("Generate metadata...")
+        logger.info("KW args %s", kwargs)
 
         self._update_check_settings(kwargs)
         self._update_globalconfig_from_settings()
         _check_global_config(self.config)
+        obj = self._check_obj_if_file(obj)
         self._establish_pwd_rootpath()
         self._validate_content_key()
         self._update_fmt_flag()
@@ -658,7 +710,7 @@ class ExportData:
 
         return deepcopy(self._metadata)
 
-    def export(self, obj, **kwargs) -> str:
+    def export(self, obj, return_symlink=False, **kwargs) -> str:
         """Export data objects of 'known' type to FMU storage solution with metadata.
 
         This function will also collect the data spesific class metadata. For "classic"
@@ -670,13 +722,15 @@ class ExportData:
 
         Args:
             obj: XTGeo instance, a Pandas Dataframe instance or other supported object.
+            return_symlink: If fmu_context is 'case_symlink_realization' then the link
+                adress will be returned if this is True; otherwise the physical file
+                path will be returned.
             **kwargs: For other arguments, see ExportData() input keys. If they
                 exist both places, this function will override!
 
         Returns:
             String: full path to exported item.
         """
-
         self.generate_metadata(obj, compute_md5=False, **kwargs)
         metadata = self._metadata
 
@@ -689,11 +743,11 @@ class ExportData:
         else:
             useflag = self._usefmtflag
 
+        obj = self._check_obj_if_file(obj)
         logger.info("Export to file and compute MD5 sum, using flag: <%s>", useflag)
         outfile, md5 = export_file_compute_checksum_md5(
             obj, outfile, outfile.suffix, flag=useflag
         )
-
         # inject md5 checksum in metadata
         metadata["file"]["checksum_md5"] = md5
 
@@ -701,9 +755,21 @@ class ExportData:
         logger.info("Actual file is:   %s", outfile)
         logger.info("Metadata file is: %s", metafile)
 
+        # generate symlink if requested
+        outfile_target = None
+        if metadata["file"].get("absolute_path_symlink"):
+            outfile_target = Path(metadata["file"]["absolute_path_symlink"])
+            outfile_source = Path(metadata["file"]["absolute_path"])
+            create_symlink(outfile_source, outfile_target)
+            metafile_target = outfile_target.parent / ("." + str(outfile.name) + ".yml")
+            create_symlink(metafile, metafile_target)
+
         self._metadata = metadata
 
-        return str(outfile)
+        if return_symlink and outfile_target:
+            return str(outfile_target)
+        else:
+            return str(outfile)
 
 
 # ######################################################################################
@@ -1140,7 +1206,7 @@ class AggregatedData:
             "masterdata": self.source_metadata[0]["masterdata"],
             "model": self.source_metadata[0]["fmu"]["model"],
         }
-        etemp = ExportData(config=fakeconfig)
+        etemp = ExportData(config=fakeconfig, name=self.name)
         etempmeta = etemp.generate_metadata(obj, compute_md5=compute_md5)
 
         template["tracklog"] = etempmeta["tracklog"]
@@ -1153,8 +1219,8 @@ class AggregatedData:
             template["data"]["name"] = self.name
         if self.tagname:
             template["data"]["tagname"] = self.tagname
-
-        template["data"]["bbox"] = etempmeta["data"]["bbox"]
+        if etempmeta["data"].get("bbox"):
+            template["data"]["bbox"] = etempmeta["data"]["bbox"]
 
         self._metadata = template
 
